/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    colourPalette

Description

\*---------------------------------------------------------------------------*/

#include "colourPalette.H"
#include "IOstreams.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

// Construct from components
colourPalette::colourPalette(const List<colour>& colours)
:
    List<colour>(colours)
{}


// Construct given number of colours
colourPalette::colourPalette(const label n)
:
    List<colour>(n)
{}


// Construct from Istream
colourPalette::colourPalette(Istream& is)
:
    List<colour>(is)
{}


// * * * * * * * * * * * * * * * Member Operators  * * * * * * * * * * * * * //

const colour& colourPalette::operator()(const word& colName) const
{
    for (label i=0; i<size(); i++)
    {
        if (operator[](i).name() == colName)
        {
            return operator[](i);
        }
    }

    FatalErrorIn("colourPalette::operator()(const word& colName) const")
        << "cannot find colour " << colName
        << abort(FatalError);

    return operator[](0);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
