/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    line2D

Description
    2D line.

SourceFiles
    line2DI.H
    line2D.C

\*---------------------------------------------------------------------------*/

#ifndef line2D_H
#define line2D_H

#include "shape2D.H"
#include "point2D.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class Gstream;
class Istream;
class Ostream;

/*---------------------------------------------------------------------------*\
                           Class line2D Declaration
\*---------------------------------------------------------------------------*/

class line2D
:
    public virtual shape2D
{

    // Private data

        point2D sttpoint;
        point2D endpoint;

public:

    // Constructors

        //- Construct from components
        inline line2D(const point2D&, const point2D&);

        //- Construct from Istream
        line2D(Istream&);


    // Destructor

        virtual ~line2D();


    // Member Functions

        // Access

            inline const point2D& start() const;
            inline const point2D& end() const;


        //- Draw
        virtual void draw(Gstream&) const;

        //- Write
        virtual void write(Ostream&) const;


    // Member Operators

        inline void operator=(const line2D&);


    // Friend Operators

        inline friend Gstream& operator<<(Gstream&, const line2D&);


    // IOstream Operators

        inline friend Ostream& operator<<(Ostream&, const line2D&);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "line2DI.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
