/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description

\*---------------------------------------------------------------------------*/

#include "GenSGSStress.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace LESmodels
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

// from components
GenSGSStress::GenSGSStress
(
    const volVectorField& U,
    const surfaceScalarField& phi,
    transportModel& transport
)
:
    LESmodel(word("GenSGSStress"), U, phi, transport),

    ce_(LESmodelProperties().lookup("ce")),
    couplingFactor_(0.0),

    B_
    (
        IOobject
        (
            "B",
            runTime_.timeName(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_
    ),

    nuSgs_
    (
        IOobject
        (
            "nuSgs",
            runTime_.timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        nu(),
        B_.boundaryField().types()
    )
{
    if (LESmodelProperties().found("couplingFactor"))
    {
        LESmodelProperties().lookup("couplingFactor") >> couplingFactor_;

        if (couplingFactor_ < 0.0 || couplingFactor_ > 1.0)
        {
            FatalErrorIn
            (
                "GenSGSStress::GenSGSStress"
                "(const volVectorField& U, const surfaceScalarField& phi,"
                "transportModel& lamTransportModel)"
            )   << "couplingFactor = " << couplingFactor_
                << " is not in range 0 - 1"
                << exit(FatalError);
        }
    }
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

tmp<fvVectorMatrix> GenSGSStress::divB
(
    volVectorField& U
) const
{
    if (couplingFactor_ > 0.0)
    {
        return
        (
            fvc::div(B_ + couplingFactor_*nuSgs_*fvc::grad(U))
          + fvc::laplacian
            (
                (1.0 - couplingFactor_)*nuSgs_, U, "laplacian(nuEff,U)"
            )
          - fvm::laplacian(nuEff(), U)
        );
    }
    else
    {
        return
        (
            fvc::div(B_)
          + fvc::laplacian(nuSgs_, U, "laplacian(nuEff,U)")
          - fvm::laplacian(nuEff(), U)
        );
    }
}


bool GenSGSStress::read()
{
    if (LESmodel::read())
    {
        LESmodelProperties().lookup("ce") >> ce_;

        LESmodelProperties().lookup("couplingFactor") >> couplingFactor_;

        if (couplingFactor_ < 0.0 || couplingFactor_ > 1.0)
        {
            FatalErrorIn("GenSGSStress::read()")
                << "couplingFactor = " << couplingFactor_
                << " is not in range 0 - 1"
                << exit(FatalError);
        }

        return true;
    }
    else
    {
        return false;
    }
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace LESmodels
} // End namespace Foam

// ************************************************************************* //
