/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description

\*---------------------------------------------------------------------------*/

#include "matchPoints.H"
#include "SortableList.H"
#include "ListOps.H"

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::matchPoints
(
    const pointField& pts0,
    const pointField& pts1,
    const scalarField& tols,
    const bool verbose,
    labelList& from0To1,
    const point& origin
)
{
    from0To1.setSize(pts0.size());
    from0To1 = -1;

    bool fullMatch = true;

    SortableList<scalar> pts0Mag(mag(pts0 - origin));

    SortableList<scalar> pts1Mag(mag(pts1 - origin));

    forAll(pts0Mag, i)
    {
        scalar dist0 = pts0Mag[i];

        label face0I = pts0Mag.indices()[i];

        scalar tol = tols[face0I];

        label startI = findLower(pts1Mag, 0.99999*dist0 - tol);

        if (startI == -1)
        {
            startI = 0;
        }

        label face1I = -1;

        // Go through range of equal mag and find equal vector.
        for
        (
            label j = startI;
            (
                (j < pts1Mag.size())
             && (pts1Mag[j] < 1.00001*dist0 + tol)
            );
            j++
        )
        {
            label faceI = pts1Mag.indices()[j];

            // Compare actual vectors
            if (mag(pts0[face0I] - pts1[faceI]) < tol)
            {
                face1I = faceI;

                break;
            }
        }

        if (face1I == -1)
        {
            if (verbose)
            {
                Info<< "Cannot find point in pts1 matching point " << face0I
                    << " coord:" << pts0[face0I]
                    << " in pts0 when using tolerance " << tol << endl;

                // Go through range of equal mag and find equal vector.
                Info<< "Searching started from:" << startI << " in pts1"
                    << endl;
                for
                (
                    label j = startI;
                    (
                        (j < pts1Mag.size())
                     && (pts1Mag[j] < dist0 + tol)
                    );
                    j++
                )
                {
                    label faceI = pts1Mag.indices()[j];

                    Info<< "Compared coord:" << pts1[faceI]
                        << " with difference to point "
                        << mag(pts1[faceI] - pts0[face0I]) << endl;
                }
            }

            fullMatch = false;
        }

        from0To1[face0I] = face1I;
    }

    return fullMatch;
}

// ************************************************************************* //
