/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description

\*---------------------------------------------------------------------------*/

#include "PrimitivePatch.H"
#include "demandDrivenData.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

template<class Face, template<class> class FaceList, class PointField>
void PrimitivePatch<Face, FaceList, PointField>::clearGeom()
{
    if (debug)
    {
        Info<< "PrimitivePatch<Face, FaceList, PointField>::clearGeom() : "
            << "clearing geometric data"
            << endl;
    }

    deleteDemandDrivenData(localPointsPtr_);
    deleteDemandDrivenData(faceNormalsPtr_);
    deleteDemandDrivenData(pointNormalsPtr_);
}


template<class Face, template<class> class FaceList, class PointField>
void PrimitivePatch<Face, FaceList, PointField>::clearTopology()
{
    if (debug)
    {
        Info<< "PrimitivePatch<Face, FaceList, PointField>::clearTopology() : "
            << "clearing patch addressing"
            << endl;
    }

    // group created and destroyed together
    if (edgesPtr_ && faceFacesPtr_ && edgeFacesPtr_ && faceEdgesPtr_)
    {
        delete edgesPtr_;
        edgesPtr_ = NULL;

        delete faceFacesPtr_;
        faceFacesPtr_ = NULL;

        delete edgeFacesPtr_;
        edgeFacesPtr_ = NULL;

        delete faceEdgesPtr_;
        faceEdgesPtr_ = NULL;
    }

    deleteDemandDrivenData(boundaryPointsPtr_);

    deleteDemandDrivenData(pointEdgesPtr_);
    deleteDemandDrivenData(pointFacesPtr_);

    deleteDemandDrivenData(edgeLoopsPtr_);
}


template<class Face, template<class> class FaceList, class PointField>
void PrimitivePatch<Face, FaceList, PointField>::clearPatchMeshAddr()
{
    if (debug)
    {
        Info<< "PrimitivePatch<Face, FaceList, PointField>::"
            << "clearPatchMeshAddr() : "
            << "clearing patch-mesh addressing"
            << endl;
    }

    deleteDemandDrivenData(meshPointsPtr_);
    deleteDemandDrivenData(meshPointMapPtr_);
    deleteDemandDrivenData(localFacesPtr_);
}


template<class Face, template<class> class FaceList, class PointField>
void PrimitivePatch<Face, FaceList, PointField>::clearOut()
{
    clearGeom();
    clearTopology();
    clearPatchMeshAddr();
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
