/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "preservePatchTypes.H"
#include "polyBoundaryMeshEntries.H"
#include "dictionary.H"

// * * * * * * * * * * * * * * * Global Functions  * * * * * * * * * * * * * //

void Foam::preservePatchTypes
(
    const objectRegistry& obr,
    const word& meshInstance,
    const fileName& meshDir,
    const wordList& patchNames,
    wordList& patchTypes,
    word& defaultFacesType,
    wordList& patchPhysicalTypes
)
{
    dictionary patchDictionary;

    {
        IOobject patchEntriesHeader
        (
            "boundary",
            meshInstance,
            meshDir,
            obr,
            IOobject::MUST_READ,
            IOobject::NO_WRITE,
            false
        );

        if (patchEntriesHeader.headerOk())
        {
            // Create a list of entries from the boundary file.
            polyBoundaryMeshEntries patchEntries(patchEntriesHeader);

            forAll(patchEntries, patchi)
            {
                patchDictionary.add(patchEntries[patchi]);
            }
        }
    }

    if (patchDictionary.size())
    {
        forAll(patchNames, patchi)
        {
            if (patchDictionary.found(patchNames[patchi]))
            {
                const dictionary& patchDict = 
                    patchDictionary.subDict(patchNames[patchi]);

                patchTypes[patchi] = word(patchDict.lookup("type"));

                if (patchDict.found("geometricType"))
                {
                    patchTypes[patchi] =
                        word(patchDict.lookup("geometricType"));
                }

                if (patchDict.found("physicalType"))
                {
                    patchPhysicalTypes[patchi] =
                        word(patchDict.lookup("physicalType"));
                }
            }
        }

        if (patchDictionary.found("defaultFaces"))
        {
            const dictionary& patchDict = 
                patchDictionary.subDict("defaultFaces");

            if (patchDict.found("geometricType"))
            {
                defaultFacesType = word(patchDict.lookup("geometricType"));
            }
        }
    }

    Info << nl << "Default patch type set to " << defaultFacesType << endl;
}


// ************************************************************************* //
