/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    calculatedFaPatchField

Description

SourceFiles
    calculatedFaPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef calculatedFaPatchField_H
#define calculatedFaPatchField_H

#include "faPatchField.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                      Class calculatedFaPatch Declaration
\*---------------------------------------------------------------------------*/

template<class Type>
class calculatedFaPatchField
:
    public faPatchField<Type>
{

public:

    //- Runtime type information
    TypeName("calculated");


    // Constructors

        //- Construct from patch and internal field
        calculatedFaPatchField
        (
            const faPatch&,
            const Field<Type>&
        );

        //- Construct from patch, internal field and dictionary
        calculatedFaPatchField
        (
            const faPatch&,
            const Field<Type>&,
            const dictionary&
        );

        //- Construct by mapping given patchField<Type> onto a new patch
        calculatedFaPatchField
        (
            const calculatedFaPatchField<Type>&,
            const faPatch&,
            const Field<Type>&,
            const faPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual tmp<faPatchField<Type> > clone() const
        {
            return tmp<faPatchField<Type> >
            (
                new calculatedFaPatchField<Type>(*this)
            );
        }

        //- Construct as copy setting internal field reference
        calculatedFaPatchField
        (
            const calculatedFaPatchField<Type>&,
            const Field<Type>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<faPatchField<Type> > clone(const Field<Type>& iF) const
        {
            return tmp<faPatchField<Type> >
            (
                new calculatedFaPatchField<Type>(*this, iF)
            );
        }


    // Member functions

        // Access

            //- Return true if this patch field fixes a value.
            //  Needed to check if a level has to be specified while solving
            //  Poissons equations.
            virtual bool fixesValue() const
            {
                return true;
            }


        // Evaluation functions

            //- Return the matrix diagonal coefficients corresponding to the
            //  evaluation of the value of this patchField with given weights
            virtual tmp<Field<Type> > valueInternalCoeffs
            (
                const tmp<scalarField>&
            ) const
            {
                FatalErrorIn
                (
                    "calculatedFaPatchField<Type>::"
                    "valueInternalCoeffs(const tmp<scalarField>&) const"
                )   << "valueInternalCoeffs cannot be called for a "
                    << "calculatedFaPatchField.  You are probably trying to "
                    << "solve for a field with calculated boundary conditions."
                    << abort(FatalError);
                return *this;
            }

            //- Return the matrix source coefficients corresponding to the
            //  evaluation of the value of this patchField with given weights
            virtual tmp<Field<Type> > valueBoundaryCoeffs
            (
                const tmp<scalarField>&
            ) const
            {
                FatalErrorIn
                (
                    "calculatedFaPatchField<Type>::"
                    "valueBoundaryCoeffs(const tmp<scalarField>&) const"
                )   << "valueBoundaryCoeffs cannot be called for a "
                    << "calculatedFaPatchField.  You are probably trying to "
                    << "solve for a field with calculated boundary conditions."
                    << abort(FatalError);
                return *this;
            }

            //- Return the matrix diagonal coefficients corresponding to the
            //  evaluation of the gradient of this patchField
            tmp<Field<Type> > gradientInternalCoeffs() const
            {
                FatalErrorIn
                (
                    "calculatedFaPatchField<Type>::"
                    "gradientInternalCoeffs() const"
                )   << "gradientInternalCoeffs cannot be called for a "
                    << "calculatedFaPatchField.  You are probably trying to "
                    << "solve for a field with calculated boundary conditions."
                    << abort(FatalError);
                return *this;
            }

            //- Return the matrix source coefficients corresponding to the
            //  evaluation of the gradient of this patchField
            tmp<Field<Type> > gradientBoundaryCoeffs() const
            {
                FatalErrorIn
                (
                    "calculatedFaPatchField<Type>::"
                    "gradientBoundaryCoeffs() const"
                )   << "gradientBoundaryCoeffs cannot be called for a "
                    << "calculatedFaPatchField.  You are probably trying to "
                    << "solve for a field with calculated boundary conditions."
                    << abort(FatalError);
                return *this;
            }


        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "calculatedFaPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
