/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    symmetryFvPatchField

Description

SourceFiles
    symmetryFvPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef symmetryFvPatchField_H
#define symmetryFvPatchField_H

#include "basicSymmetryFvPatchField.H"
#include "symmetryFvPatch.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                         Class symmetryFvPatch Declaration
\*---------------------------------------------------------------------------*/

template<class Type>
class symmetryFvPatchField
:
    public basicSymmetryFvPatchField<Type>
{

public:

    //- Runtime type information
    TypeName(symmetryFvPatch::typeName_());


    // Constructors

        //- Construct from patch and internal field
        symmetryFvPatchField
        (
            const fvPatch&,
            const Field<Type>&
        );

        //- Construct from patch, internal field and dictionary
        symmetryFvPatchField
        (
            const fvPatch&,
            const Field<Type>&,
            const dictionary&
        );

        //- Construct by mapping given symmetryFvPatchField onto a new patch
        symmetryFvPatchField
        (
            const symmetryFvPatchField<Type>&,
            const fvPatch&,
            const Field<Type>&,
            const fvPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchField<Type> > clone() const
        {
            return tmp<fvPatchField<Type> >
            (
                new symmetryFvPatchField<Type>(*this)
            );
        }

        //- Construct as copy setting internal field reference
        symmetryFvPatchField
        (
            const symmetryFvPatchField<Type>&,
            const Field<Type>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchField<Type> > clone(const Field<Type>& iF) const
        {
            return tmp<fvPatchField<Type> >
            (
                new symmetryFvPatchField<Type>(*this, iF)
            );
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "symmetryFvPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
