/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    sphericalCS

Description
    Spherical polar coordinate system.

SourceFiles
    sphericalCS.C

\*---------------------------------------------------------------------------*/

#ifndef sphericalCS_H
#define sphericalCS_H

#include "cartesianCS.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class sphericalCS Declaration
\*---------------------------------------------------------------------------*/

class sphericalCS
:
    public cartesianCS
{

public:

    //- Runtime type information
    TypeName("spherical");


    // Constructors

        //- Construct from components
        sphericalCS
        (
            const word& name,
            const vector& origin,
            const vector& axis,
            const vector& direction
        );

        //- Construct from origin and rotation angles
        sphericalCS
        (
            const word& name,
            const vector& origin,
            const coordinateRotation& cr
        );


        //- Construct from dictionary
        sphericalCS(const word& name, const dictionary& dict);


    // Member Functions

        //- Convert from local coordinate system to the global Cartesian system
        virtual vector toGlobal(const vector& localV) const;
        virtual tmp<vectorField> toGlobal(const vectorField& localV) const;

        //- Convert from global Cartesian system to the local coordinate system 
        virtual vector toLocal(const vector& globalV) const;
        virtual tmp<vectorField> toLocal(const vectorField& globalV) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
