/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    Probe

Description
    Probes a volTypeField on call to write() and writes the values into a file.

SourceFiles
    Probe.C

\*---------------------------------------------------------------------------*/

#ifndef Probe_H
#define Probe_H

#include "volFieldsFwd.H"
#include "OFstream.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class Probe Declaration
\*---------------------------------------------------------------------------*/

template<class volTypeField>
class Probe
{
    // Private data

        const volTypeField& field_;
        vectorField probeLocations_;
        labelList cellList_;
        bool probes_;
        OFstream* probeFilePtr_;


    // Private Member Functions

        //- Construct as copy (not implemented)
        Probe(const Probe&);

        //- Assignment (not implemented)
        Probe& operator=(const Probe&);


public:

    // Constructors

        //- Construct from components
        Probe(const dictionary&, const volTypeField&);


    // Member Functions

        void write();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "Probe.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
