/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    GlobalProcessorTetPointPatchField

Description

SourceFiles
    GlobalProcessorTetPointPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef GlobalProcessorTetPointPatchField_H
#define GlobalProcessorTetPointPatchField_H

#include "CoupledTetPointPatchField.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                Class GlobalProcessorTetPointPatchField Declaration
\*---------------------------------------------------------------------------*/

template
<
    template<class> class PatchField,
    class PointPatch,
    class GlobalProcessorPointPatch,
    class Type
>
class GlobalProcessorTetPointPatchField
:
    public CoupledTetPointPatchField<PatchField, PointPatch, Type>
{
    // Private data

        //- Local reference to processorPoint patch
        const GlobalProcessorPointPatch& procPatch_;


    // Private member functions

        //- Reduce the field and extract the local values
        template<class Type2>
        tmp<Field<Type2> > reduceExtractPoint(const tmp<Field<Type2> >&) const;

        template<class Type2>
        tmp<Field<Type2> > reduceExtractEdge(const tmp<Field<Type2> >&) const;

        template<class Type2>
        void addFieldTempl(Field<Type2>&) const;


public:

    //- Runtime type information
    TypeName("globalProcessor");


    // Constructors

        //- Construct from patch and internal field
        GlobalProcessorTetPointPatchField
        (
            const PointPatch&,
            const Field<Type>&
        );

        //- Construct from patch, internal field and dictionary
        GlobalProcessorTetPointPatchField
        (
            const PointPatch&,
            const Field<Type>&,
            const dictionary&
        );

        //- Construct by mapping given patchField<Type> onto a new patch
        GlobalProcessorTetPointPatchField
        (
            const GlobalProcessorTetPointPatchField
                <PatchField, PointPatch, GlobalProcessorPointPatch, Type>&,
            const PointPatch&,
            const Field<Type>&,
            const TetPointPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual autoPtr<PatchField<Type> > clone() const
        {
            return autoPtr<PatchField<Type> >
            (
                new GlobalProcessorTetPointPatchField
                <PatchField, PointPatch, GlobalProcessorPointPatch, Type>
                (
                    *this
                )
            );
        }

        //- Construct as copy setting internal field reference
        GlobalProcessorTetPointPatchField
        (
            const GlobalProcessorTetPointPatchField
                <PatchField, PointPatch, GlobalProcessorPointPatch, Type>&,
            const Field<Type>&
        );

        //- Construct and return a clone setting internal field reference
        virtual autoPtr<PatchField<Type> > clone(const Field<Type>& iF) const
        {
            return autoPtr<PatchField<Type> >
            (
                new GlobalProcessorTetPointPatchField
                <PatchField, PointPatch, GlobalProcessorPointPatch, Type>
                (
                    *this,
                    iF
                )
            );
        }


    // Destructor

        ~GlobalProcessorTetPointPatchField();


    // Member functions

        // Access

            //- Evaluate the patch field
            virtual void evaluate()
            {}

            //- Add field to internal field
            virtual void addField(Field<Type>&) const;

            //- Set boundary condition to matrix.  Used to save the processor
            //  patch off-diagonal coefficients
            virtual void setBoundaryCondition
            (
                Map<constraint<Type> > &
            ) const;

        // Matrix construction (completion) functionality

            virtual void addDiag(scalarField&) const;
            virtual void addSource(scalarField&) const;

            //- Add upper/lower contributions
            virtual void addUpperLower(scalarField&) const;

            virtual void eliminateUpperLower(scalarField&) const;

            //- Get the cut edge coefficients in Amul order
            virtual tmp<scalarField> cutBouCoeffs(const lduMatrix&) const;

            //- Get the cut edge coefficients in Tmul order
            virtual tmp<scalarField> cutIntCoeffs(const lduMatrix&) const;


        // Coupled interface functionality

            //- Return neighbour colouring
            virtual tmp<labelField> nbrColour
            (
                const labelField&
            ) const;

            //- Update result field based on interface functionality
            virtual void updateInterfaceMatrix
            (
                const scalarField& psiInternal,
                scalarField& result,
                const lduMatrix& m,
                const scalarField& coeffs,
                const direction cmpt
            ) const;

};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#    include "GlobalProcessorTetPointPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
