/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    ProcessorTetPointPatchField

Description

SourceFiles
    ProcessorTetPointPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef ProcessorTetPointPatchField_H
#define ProcessorTetPointPatchField_H

#include "CoupledTetPointPatchField.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                 Class ProcessorTetPointPatchField Declaration
\*---------------------------------------------------------------------------*/

template
<
    template<class> class PatchField,
    class PointPatch,
    class ProcessorPointPatch,
    class Type
>
class ProcessorTetPointPatchField
:
    public CoupledTetPointPatchField<PatchField, PointPatch, Type>
{
    // Private data

        //- Local reference to processor patch
        const ProcessorPointPatch& procPatch_;


    // Private member functions

        //- Raw field send function
        template<class Type2>
        void sendPointField(const tmp<Field<Type2> >&) const;

        template<class Type2>
        void sendEdgeField(const tmp<Field<Type2> >&) const;


        //- Raw field receive function
        template<class Type2>
        tmp<Field<Type2> > receivePointField() const;

        template<class Type2>
        tmp<Field<Type2> > receiveEdgeField() const;

        //- Templated add diagonal/source functions
        template<class Type2>
        void initAddFieldTempl(const Field<Type2>&) const;

        template<class Type2>
        void addFieldTempl(Field<Type2>&) const;



public:

    //- Runtime type information
    TypeName(ProcessorPointPatch::typeName_());


    // Constructors

        //- Construct from patch and internal field
        ProcessorTetPointPatchField
        (
            const PointPatch&,
            const Field<Type>&
        );

        //- Construct from patch, internal field and dictionary
        ProcessorTetPointPatchField
        (
            const PointPatch&,
            const Field<Type>&,
            const dictionary&
        );

        //- Construct by mapping given patchField<Type> onto a new patch
        ProcessorTetPointPatchField
        (
            const ProcessorTetPointPatchField
                <PatchField, PointPatch, ProcessorPointPatch, Type>&,
            const PointPatch&,
            const Field<Type>&,
            const TetPointPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual autoPtr<PatchField<Type> > clone() const
        {
            return autoPtr<PatchField<Type> >
            (
                new ProcessorTetPointPatchField
                <PatchField, PointPatch, ProcessorPointPatch, Type>
                (
                    *this
                )
            );
        }

        //- Construct as copy setting internal field reference
        ProcessorTetPointPatchField
        (
            const ProcessorTetPointPatchField
                <PatchField, PointPatch, ProcessorPointPatch, Type>&,
            const Field<Type>&
        );

        //- Construct and return a clone setting internal field reference
        virtual autoPtr<PatchField<Type> > clone(const Field<Type>& iF) const
        {
            return autoPtr<PatchField<Type> >
            (
                new ProcessorTetPointPatchField
                <PatchField, PointPatch, ProcessorPointPatch, Type>
                (
                    *this,
                    iF
                )
            );
        }


    // Destructor

        ~ProcessorTetPointPatchField();


    // Member functions

        // Access

            //- Evaluate the patch field
            virtual void evaluate()
            {}

            //- Init add field to internal field
            virtual void initAddField() const;

            //- Add field to internal field
            virtual void addField(Field<Type>&) const;

            //- Set boundary condition to matrix.  Used to save the processor
            //  patch off-diagonal coefficients
            virtual void setBoundaryCondition
            (
                Map<constraint<Type> > &
            ) const;


        // Matrix construction (completion) functionality

            //- Add diagonal/source contributions
            virtual void initAddDiag(const scalarField&) const;
            virtual void initAddSource(const scalarField&) const;

            virtual void addDiag(scalarField&) const;
            virtual void addSource(scalarField&) const;

            //- Add upper/lower contributions
            virtual void initAddUpperLower(const scalarField&) const;

            virtual void addUpperLower(scalarField&) const;

            virtual void eliminateUpperLower(scalarField&) const;

            //- Get the cut edge coefficients in Amul order
            virtual tmp<scalarField> cutBouCoeffs(const lduMatrix&) const;

            //- Get the cut edge coefficients in Tmul order
            virtual tmp<scalarField> cutIntCoeffs(const lduMatrix&) const;


        // Coupled interface functionality

            //- Initialise neighbour colouring transfer
            virtual void initNbrColour
            (
                const labelField&,
                const bool bufferdTransfer
            ) const;

            //- Return neighbour colouring
            virtual tmp<labelField> nbrColour
            (
                const labelField&
            ) const;

            //- Initialise neighbour matrix update
            virtual void initInterfaceMatrixUpdate
            (
                const scalarField& psiInternal,
                scalarField& result,
                const lduMatrix& m,
                const scalarField& coeffs,
                const direction cmpt,
                const bool bufferdTransfer
            ) const;

            //- Update result field based on interface functionality
            virtual void updateInterfaceMatrix
            (
                const scalarField& psiInternal,
                scalarField& result,
                const lduMatrix& m,
                const scalarField& coeffs,
                const direction cmpt
            ) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#    include "ProcessorTetPointPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
