/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description
    Non-orthogonality-based motion diffusion.

\*---------------------------------------------------------------------------*/

#include "deformationEnergyDiff.H"
#include "addToRunTimeSelectionTable.H"
#include "elementFields.H"
#include "cellQuality.H"
#include "tetFec.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(deformationEnergyDiff, 0);
    addToRunTimeSelectionTable(motionDiff, deformationEnergyDiff, dictionary);
}

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

// Construct from components
Foam::deformationEnergyDiff::deformationEnergyDiff
(
    const tetDecompositionMotionSolver& mSolver
)
:
    motionDiff(mSolver),
    gamma_
    (
	IOobject
	(
	    "gamma",
	    tetMesh().time().timeName(),
	    tetMesh()(),
	    IOobject::NO_READ,
	    IOobject::NO_WRITE
        ),
	tetMesh(),
	dimensionedScalar("1.0", dimless, 1.0)
    )
{
    ITstream is = mSolver.lookup("diffusion");

    token t;
    is.read(t);

    exponent_ = readInt(is);

    Info << "Value of exponent for deformation energy based motion diffusion: " 
	 << exponent_ << endl;

    mSolver.storeTotDisplacement();

    correct();
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::deformationEnergyDiff::~deformationEnergyDiff()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::deformationEnergyDiff::correct()
{
    gamma_.internalField() = 
	pow(mSolver().totDeformationEnergy()().internalField(), exponent_)
      + SMALL;

    gamma_.internalField() /= max(gamma_.internalField());
}

// ************************************************************************* //
