/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    hThermo

SourceFiles
    hThermo.C

\*---------------------------------------------------------------------------*/

#ifndef hThermo_H
#define hThermo_H

#include "basicThermo.H"
#include "basicMixture.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class hThermo Declaration
\*---------------------------------------------------------------------------*/

template<class MixtureType>
class hThermo
:
    public basicThermo,
    public MixtureType
{
    // Private data

        volScalarField h_;


    // Private member functions

        void calculate();

        //- Construct as copy (not implemented)
        hThermo(const hThermo<MixtureType>&);


public:

    //- Runtime type information
    TypeName("hThermo");


    // Constructors

        //- Construct from mesh
        hThermo(const fvMesh&);


    // Destructor

        ~hThermo();


    // Member functions

        //- Return the compostion of the combustion mixture
        basicMixture& composition()
        {
            return *this;
        }

        //- Return the compostion of the combustion mixture
        const basicMixture& composition() const
        {
            return *this;
        }

        //- Update properties
        void correct();


        // Access to thermodynamic state variables

            //- Enthalpy [J/kg]
            //  Non-const access allowed for transport equations
            volScalarField& h()
            {
                return h_;
            }

            //- Enthalpy [J/kg]
            const volScalarField& h() const
            {
                return h_;
            }


        // Fields derived from thermodynamic state variables

            //- Enthalpy for cell-set [J/kg]
            tmp<scalarField> h
            (
                const scalarField& T,
                const labelList& cells
            ) const;

            //- Enthalpy for patch [J/kg]
            tmp<scalarField> h
            (
                const scalarField& T,
                const label patchi
            ) const;

            //- Heat capacity at constant pressure for patch [J/kg/K]
            tmp<scalarField> Cp(const scalarField& T, const label patchi) const;

            //- Heat capacity at constant pressure [J/kg/K]
            tmp<volScalarField> Cp() const;

            //- Heat capacity at constant volume [J/kg/K]
            tmp<volScalarField> Cv() const;


        //- Read thermophysicalProperties dictionary
        bool read();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

#ifdef NoRepository
#   include "hThermo.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
