/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Description

\*---------------------------------------------------------------------------*/

#include "chemistryReader.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineTypeNameAndDebug(chemistryReader, 0);
    defineRunTimeSelectionTable(chemistryReader, dictionary);
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

Foam::autoPtr<Foam::chemistryReader> Foam::chemistryReader::New
(
    const dictionary& thermoDict
)
{
    // Let the chemistry reader type default to CHEMKIN
    // for backward compatability
    word chemistryReaderTypeName("chemkinReader");

    // otherwise use the specified reader
    if (thermoDict.found("chemistryReader"))
    {
        thermoDict.lookup("chemistryReader") >> chemistryReaderTypeName;
    }

    Info<< "Selecting chemistryReader " << chemistryReaderTypeName << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_
        ->find(chemistryReaderTypeName);

    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorIn
        (
            "chemistryReader::New(const dictionary& thermoDict)"
        )   << "Unknown chemistryReader type "
            << chemistryReaderTypeName << endl << endl
            << "Valid  chemistryReaders are : " << endl
            << dictionaryConstructorTablePtr_->toc()
            << exit(FatalError);
    }

    return autoPtr<chemistryReader>(cstrIter()(thermoDict));
}


// ************************************************************************* //
