/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "nutStandardWallFunctionFvPatchScalarField.H"
#include "turbulenceModel.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace turbulenceModels
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

nutStandardWallFunctionFvPatchScalarField::nutStandardWallFunctionFvPatchScalarField
(
    const fvPatch& p,
    const scalarField& iF
)
:
    fixedValueFvPatchScalarField(p, iF)
{}


nutStandardWallFunctionFvPatchScalarField::nutStandardWallFunctionFvPatchScalarField
(
    const nutStandardWallFunctionFvPatchScalarField& ptf,
    const fvPatch& p,
    const scalarField& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchScalarField(ptf, p, iF, mapper)
{}


nutStandardWallFunctionFvPatchScalarField::nutStandardWallFunctionFvPatchScalarField
(
    const fvPatch& p,
    const scalarField& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchScalarField(p, iF, dict)
{}


nutStandardWallFunctionFvPatchScalarField::nutStandardWallFunctionFvPatchScalarField
(
    const nutStandardWallFunctionFvPatchScalarField& tppsf,
    const scalarField& iF
)
:
    fixedValueFvPatchScalarField(tppsf, iF)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void nutStandardWallFunctionFvPatchScalarField::evaluate()
{
    const turbulenceModel& rasModel
        = db().lookupObject<turbulenceModel>("turbulenceProperties");

    scalar kappa = rasModel.kappa();
    scalar E = rasModel.E();
    scalar yPlusLam = rasModel.E();

    const scalarField& ry = patch().deltaCoeffs();

    const fvPatchVectorField& U = lookupPatchField<volVectorField, vector>("U");
    scalarField magUp = mag(U.patchInternalField() - U);

    const scalarField& nuw = lookupPatchField<volScalarField, scalar>("nu");
    scalarField& nutw = *this;

    scalarField magFaceGradU = mag(U.snGrad());

    forAll(nutw, facei)
    {
        scalar magUpara = magUp[facei];

        scalar kappaRe = kappa*magUpara/(nuw[facei]*ry[facei]);

        scalar yPlus = yPlusLam;
        scalar ryPlusLam = 1.0/yPlus;

        int iter = 0;
        scalar yPlusLast = 0.0;

        do
        {
            yPlusLast = yPlus;
            yPlus = (kappaRe + yPlus)/(1.0 + log(E*yPlus));

        } while(mag(ryPlusLam*(yPlus - yPlusLast)) > 0.01 && ++iter < 10 );
 
        if (yPlus > yPlusLam)
        {
            nutw[facei] = nuw[facei]*(yPlus*kappa/log(E*yPlus) - 1);
        }
        else
        {
            nutw[facei] = 0.0;
        }
    }
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchScalarField, nutStandardWallFunctionFvPatchScalarField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace turbulenceModels
} // End namespace Foam

// ************************************************************************* //
